import json
import grpc
import jwt
import datetime
from solidity_parser import parser

from .stubs import sidechain_eth_pb2, sidechain_eth_pb2_grpc
from elastos_adenine.settings import REQUEST_TIMEOUT, TOKEN_EXPIRATION


class SidechainEth:

    def __init__(self, host, port, production):
        if not production:
            self._channel = grpc.insecure_channel('{}:{}'.format(host, port))
        else:
            credentials = grpc.ssl_channel_credentials()
            self._channel = grpc.secure_channel('{}:{}'.format(host, port), credentials)

        self.stub = sidechain_eth_pb2_grpc.SidechainEthStub(self._channel)

    def close(self):
        self._channel.close()

    def deploy_eth_contract(self, api_key, did, network, eth_account_address, eth_private_key, eth_gas, filename):
        with open(filename, 'r') as myfile:
            contract_source = myfile.read()
        contract_metadata = parser.parse_file(filename)
        contract_name = contract_metadata['children'][1]['name']
        
        jwt_info = {
            'network': network,
            'eth_account_address': eth_account_address,
            'eth_private_key': eth_private_key,
            'eth_gas': eth_gas,
            'contract_source': contract_source,
            'contract_name': contract_name,
        }

        jwt_token = jwt.encode({
            'jwt_info': jwt_info,
            'exp': datetime.datetime.utcnow() + datetime.timedelta(hours=TOKEN_EXPIRATION)
        }, api_key, algorithm='HS256')

        response = self.stub.DeployEthContract(sidechain_eth_pb2.Request(input=jwt_token), timeout=REQUEST_TIMEOUT, metadata=[('did', did)])
        
        if response.status:
            output = jwt.decode(response.output, key=api_key, algorithms=['HS256']).get('jwt_info')
            result = {
                'output': json.dumps(output),
                'status_message': response.status_message,
                'status': response.status
            }
            return result
        else:
            result = {
                'output': '',
                'status_message': response.status_message,
                'status': response.status
            }
            return result

    def watch_eth_contract(self, api_key, did, network, contract_address, contract_name, contract_code_hash):
        jwt_info = {
            'network': network,
            'contract_address': contract_address,
            'contract_name': contract_name,
            'contract_code_hash': contract_code_hash,
        }

        jwt_token = jwt.encode({
            'jwt_info': jwt_info,
            'exp': datetime.datetime.utcnow() + datetime.timedelta(hours=TOKEN_EXPIRATION)
        }, api_key, algorithm='HS256')

        response = self.stub.WatchEthContract(sidechain_eth_pb2.Request(input=jwt_token), timeout=REQUEST_TIMEOUT, metadata=[('did', did)])
        
        if response.status:
            output = jwt.decode(response.output, key=api_key, algorithms=['HS256']).get('jwt_info')
            result = {
                'output': json.dumps(output),
                'status_message': response.status_message,
                'status': response.status
            }
            return result
        else:
            result = {
                'output': '',
                'status_message': response.status_message,
                'status': response.status
            }
            return result



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from . import sidechain_eth_pb2 as sidechain__eth__pb2


class SidechainEthStub(object):
  """The service definition.
  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.DeployEthContract = channel.unary_unary(
        '/sidechain_eth.SidechainEth/DeployEthContract',
        request_serializer=sidechain__eth__pb2.Request.SerializeToString,
        response_deserializer=sidechain__eth__pb2.Response.FromString,
        )
    self.WatchEthContract = channel.unary_unary(
        '/sidechain_eth.SidechainEth/WatchEthContract',
        request_serializer=sidechain__eth__pb2.Request.SerializeToString,
        response_deserializer=sidechain__eth__pb2.Response.FromString,
        )


class SidechainEthServicer(object):
  """The service definition.
  """

  def DeployEthContract(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def WatchEthContract(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_SidechainEthServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'DeployEthContract': grpc.unary_unary_rpc_method_handler(
          servicer.DeployEthContract,
          request_deserializer=sidechain__eth__pb2.Request.FromString,
          response_serializer=sidechain__eth__pb2.Response.SerializeToString,
      ),
      'WatchEthContract': grpc.unary_unary_rpc_method_handler(
          servicer.WatchEthContract,
          request_deserializer=sidechain__eth__pb2.Request.FromString,
          response_serializer=sidechain__eth__pb2.Response.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'sidechain_eth.SidechainEth', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: health_check.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='health_check.proto',
  package='health_check',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x12health_check.proto\x12\x0chealth_check\"%\n\x12HealthCheckRequest\x12\x0f\n\x07service\x18\x01 \x01(\t\"\xa7\x01\n\x13HealthCheckResponse\x12?\n\x06status\x18\x01 \x01(\x0e\x32/.health_check.HealthCheckResponse.ServingStatus\"O\n\rServingStatus\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07SERVING\x10\x01\x12\x0f\n\x0bNOT_SERVING\x10\x02\x12\x13\n\x0fSERVICE_UNKNOWN\x10\x03\x32\xa6\x01\n\x06Health\x12L\n\x05\x43heck\x12 .health_check.HealthCheckRequest\x1a!.health_check.HealthCheckResponse\x12N\n\x05Watch\x12 .health_check.HealthCheckRequest\x1a!.health_check.HealthCheckResponse0\x01\x62\x06proto3')
)



_HEALTHCHECKRESPONSE_SERVINGSTATUS = _descriptor.EnumDescriptor(
  name='ServingStatus',
  full_name='health_check.HealthCheckResponse.ServingStatus',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SERVING', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NOT_SERVING', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SERVICE_UNKNOWN', index=3, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=164,
  serialized_end=243,
)
_sym_db.RegisterEnumDescriptor(_HEALTHCHECKRESPONSE_SERVINGSTATUS)


_HEALTHCHECKREQUEST = _descriptor.Descriptor(
  name='HealthCheckRequest',
  full_name='health_check.HealthCheckRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='service', full_name='health_check.HealthCheckRequest.service', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=36,
  serialized_end=73,
)


_HEALTHCHECKRESPONSE = _descriptor.Descriptor(
  name='HealthCheckResponse',
  full_name='health_check.HealthCheckResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='health_check.HealthCheckResponse.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _HEALTHCHECKRESPONSE_SERVINGSTATUS,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=76,
  serialized_end=243,
)

_HEALTHCHECKRESPONSE.fields_by_name['status'].enum_type = _HEALTHCHECKRESPONSE_SERVINGSTATUS
_HEALTHCHECKRESPONSE_SERVINGSTATUS.containing_type = _HEALTHCHECKRESPONSE
DESCRIPTOR.message_types_by_name['HealthCheckRequest'] = _HEALTHCHECKREQUEST
DESCRIPTOR.message_types_by_name['HealthCheckResponse'] = _HEALTHCHECKRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

HealthCheckRequest = _reflection.GeneratedProtocolMessageType('HealthCheckRequest', (_message.Message,), {
  'DESCRIPTOR' : _HEALTHCHECKREQUEST,
  '__module__' : 'health_check_pb2'
  # @@protoc_insertion_point(class_scope:health_check.HealthCheckRequest)
  })
_sym_db.RegisterMessage(HealthCheckRequest)

HealthCheckResponse = _reflection.GeneratedProtocolMessageType('HealthCheckResponse', (_message.Message,), {
  'DESCRIPTOR' : _HEALTHCHECKRESPONSE,
  '__module__' : 'health_check_pb2'
  # @@protoc_insertion_point(class_scope:health_check.HealthCheckResponse)
  })
_sym_db.RegisterMessage(HealthCheckResponse)



_HEALTH = _descriptor.ServiceDescriptor(
  name='Health',
  full_name='health_check.Health',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=246,
  serialized_end=412,
  methods=[
  _descriptor.MethodDescriptor(
    name='Check',
    full_name='health_check.Health.Check',
    index=0,
    containing_service=None,
    input_type=_HEALTHCHECKREQUEST,
    output_type=_HEALTHCHECKRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Watch',
    full_name='health_check.Health.Watch',
    index=1,
    containing_service=None,
    input_type=_HEALTHCHECKREQUEST,
    output_type=_HEALTHCHECKRESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_HEALTH)

DESCRIPTOR.services_by_name['Health'] = _HEALTH

# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from . import node_rpc_pb2 as node__rpc__pb2


class NodeRpcStub(object):
  """The service definition.
  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.RpcMethod = channel.unary_unary(
        '/node_rpc.NodeRpc/RpcMethod',
        request_serializer=node__rpc__pb2.Request.SerializeToString,
        response_deserializer=node__rpc__pb2.Response.FromString,
        )


class NodeRpcServicer(object):
  """The service definition.
  """

  def RpcMethod(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_NodeRpcServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'RpcMethod': grpc.unary_unary_rpc_method_handler(
          servicer.RpcMethod,
          request_deserializer=node__rpc__pb2.Request.FromString,
          response_serializer=node__rpc__pb2.Response.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'node_rpc.NodeRpc', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from . import common_pb2 as common__pb2


class CommonStub(object):
  """The common service definition.
  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.GenerateAPIRequest = channel.unary_unary(
        '/common.Common/GenerateAPIRequest',
        request_serializer=common__pb2.Request.SerializeToString,
        response_deserializer=common__pb2.Response.FromString,
        )
    self.GetAPIKey = channel.unary_unary(
        '/common.Common/GetAPIKey',
        request_serializer=common__pb2.Request.SerializeToString,
        response_deserializer=common__pb2.Response.FromString,
        )


class CommonServicer(object):
  """The common service definition.
  """

  def GenerateAPIRequest(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetAPIKey(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_CommonServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'GenerateAPIRequest': grpc.unary_unary_rpc_method_handler(
          servicer.GenerateAPIRequest,
          request_deserializer=common__pb2.Request.FromString,
          response_serializer=common__pb2.Response.SerializeToString,
      ),
      'GetAPIKey': grpc.unary_unary_rpc_method_handler(
          servicer.GetAPIKey,
          request_deserializer=common__pb2.Request.FromString,
          response_serializer=common__pb2.Response.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'common.Common', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



import pytest
import sys
from decouple import config
import json
import argparse

from elastos_adenine.stubs import health_check_pb2
from elastos_adenine.health_check import HealthCheck
from elastos_adenine.common import Common
from elastos_adenine.hive import Hive
from elastos_adenine.sidechain_eth import SidechainEth
from elastos_adenine.wallet import Wallet

host = config('GRPC_SERVER_HOST')
port = config('GRPC_SERVER_PORT')
production = config('PRODUCTION', default=False, cast=bool)

network = "gmunet"
mnemonic_to_use = 'obtain pill nest sample caution stone candy habit silk husband give net'
did_to_use = 'n84dqvIK9O0LIPXi27uL0aRnoR45Exdxl218eQyPDD4lW8RPov'
api_key_to_use = ''
private_key_to_use = '1F54BCD5592709B695E85F83EBDA515971723AFF56B32E175F14A158D5AC0D99'

def test_generate_api_key():
	# Generate API Key
	global api_key_to_use
	common = Common(host, port, production)
	response = common.generate_api_request(config('SHARED_SECRET_ADENINE'), did_to_use)
	if response.status:
		api_key_to_use = response.api_key
	assert response.status==True,"Testing Generate API Key Failed"
	assert len(response.api_key) == 64,"Testing API Key length Failed"

def test_upload_and_sign():
	# Upload And Sign
	hive = Hive(host, port, production)
	response = hive.upload_and_sign(api_key_to_use, network, private_key_to_use, 'test/sample.txt')
	assert response.status==True,"Testing Upload And Sign Failed"



import json
import grpc
import jwt
import datetime

from .stubs import hive_pb2, hive_pb2_grpc
from elastos_adenine.settings import REQUEST_TIMEOUT, TOKEN_EXPIRATION


class Hive:

    def __init__(self, host, port, production):
        if not production:
            self._channel = grpc.insecure_channel('{}:{}'.format(host, port))
        else:
            credentials = grpc.ssl_channel_credentials()
            self._channel = grpc.secure_channel('{}:{}'.format(host, port), credentials)

        self.stub = hive_pb2_grpc.HiveStub(self._channel)

    def close(self):
        self._channel.close()

    def upload_and_sign(self, api_key, did, network, private_key, filename):
        with open(filename, 'rb') as myfile:
            file_content = myfile.read()

        # generate JWT token
        jwt_info = {
            'network': network,
            'privateKey': private_key
        }

        jwt_token = jwt.encode({
            'jwt_info': jwt_info,
            'exp': datetime.datetime.utcnow() + datetime.timedelta(hours=TOKEN_EXPIRATION)
        }, api_key, algorithm='HS256')

        response = self.stub.UploadAndSign(hive_pb2.Request(input=jwt_token, file_content=file_content),
                                           timeout=REQUEST_TIMEOUT, metadata=[('did', did)])

        if response.status:
            output = jwt.decode(response.output, key=api_key, algorithms=['HS256']).get('jwt_info')
            result = {
                'output': json.dumps(output),
                'status_message': response.status_message,
                'status': response.status
            }

        else:
            result = {
                'output': '',
                'status_message': response.status_message,
                'status': response.status
            }
        return result

    def verify_and_show(self, api_key, did, network, request_input):
        jwt_info = {
            'network': network,
            'msg': request_input['msg'],
            'pub': request_input['pub'],
            'sig': request_input['sig'],
            'hash': request_input['hash'],
            'privateKey': request_input['privateKey']
        }

        jwt_token = jwt.encode({
            'jwt_info': jwt_info,
            'exp': datetime.datetime.utcnow() + datetime.timedelta(hours=TOKEN_EXPIRATION)
        }, api_key, algorithm='HS256')

        response = self.stub.VerifyAndShow(hive_pb2.Request(input=jwt_token), timeout=REQUEST_TIMEOUT,
                                           metadata=[('did', did)])

        if response.status:
            result = {
                'file_content': response.file_content,
                'status_message': response.status_message,
                'status': response.status
            }
        else:
            result = {
                'output': '',
                'status_message': response.status_message,
                'status': response.status
            }
        return result






import grpc
import jwt
import datetime

from ast import literal_eval
from .stubs import node_rpc_pb2, node_rpc_pb2_grpc
from elastos_adenine.settings import REQUEST_TIMEOUT, TOKEN_EXPIRATION


class NodeRpc:

    def __init__(self, host, port, production):
        if not production:
            self._channel = grpc.insecure_channel('{}:{}'.format(host, port))
        else:
            credentials = grpc.ssl_channel_credentials()
            self._channel = grpc.secure_channel('{}:{}'.format(host, port), credentials)

        self.stub = node_rpc_pb2_grpc.NodeRpcStub(self._channel)

    def close(self):
        self._channel.close()

    # Common method for mainchain only
    def get_current_crc_council(self, api_key, did, network):
        params = {"state": "all"}
        return self.rpc_method(api_key, did, network, "mainchain", "listcurrentcrs", params)

    # Common method for mainchain only
    def get_current_crc_candidates(self, api_key, did, network):
        params = {"start": 0, "state": "all"}
        return self.rpc_method(api_key, did, network, "mainchain", "listcrcandidates", params)

    # Common method for mainchain only
    def get_current_dpos_supernodes(self, api_key, did, network):
        params = {"start": 0, "state": "all"}
        return self.rpc_method(api_key, did, network, "mainchain", "listproducers", params)

    # Common method for mainchain only
    def get_current_arbitrator_group(self, api_key, did, network):
        params = {"height": str(self.get_current_height(api_key, did, network, "mainchain"))}
        return self.rpc_method(api_key, did, network, "mainchain", "getarbitratorgroupbyheight", params)

    # Common method for mainchain only
    def get_arbitrator_group(self, api_key, did, network, height):
        params = {"height": height}
        return self.rpc_method(api_key, did, network, "mainchain", "getarbitratorgroupbyheight", params)

    # Common method for mainchain only
    def get_current_arbitrators_info(self, api_key, did, network):
        return self.rpc_method(api_key, did, network, "mainchain", "getarbitersinfo", {})

    # Common method for mainchain only
    def get_current_block_confirm(self, api_key, did, network):
        height = self.get_current_height(api_key, did, network, "mainchain")
        return self.get_block_confirm(api_key, did, network, height)

    # Common method for mainchain only
    def get_block_confirm(self, api_key, did, network, height):
        params = {"height": height, "verbosity": 1}
        return self.rpc_method(api_key, did, network, "mainchain", "getconfirmbyheight", params)

    # Common method for mainchain only
    def get_current_mining_info(self, api_key, did, network):
        return self.rpc_method(api_key, did, network, "mainchain", "getmininginfo", {})

    # Common method for mainchain, did sidechain, token and eth sidechain
    def get_current_block_info(self, api_key, did, network, chain):
        height = self.get_current_height(api_key, did, network, chain)
        return self.get_block_info(api_key, did, network, chain, height)

    # Common method for mainchain, did sidechain, token and eth sidechain
    def get_block_info(self, api_key, did, network, chain, height):
        if chain == "eth":
            method = "eth_getBlockByNumber"
            params = [hex(height), True]
        else:
            method = "getblockbyheight"
            params = {"height": str(height)}
        return self.rpc_method(api_key, did, network, chain, method, params)

    # Common method for mainchain, did sidechain, token and eth sidechain
    def get_current_balance(self, api_key, did, network, chain, address):
        if chain == "eth":
            method = "eth_getBalance"
            params = [address, "latest"]
            current_balance = self.rpc_method(api_key, did, network, chain, method, params)
            current_balance = literal_eval(current_balance) / 1000000000000000000
        else:
            method = "getreceivedbyaddress"
            params = {"address": address}
            current_balance = self.rpc_method(api_key, did, network, chain, method, params)
        return current_balance

    # Common method for mainchain, did sidechain, token and eth sidechain
    def get_current_height(self, api_key, did, network, chain):
        if chain == "eth":
            current_height = literal_eval(self.rpc_method(api_key, did, network, chain, "eth_blockNumber", {}))
        else:
            node_state = self.get_current_node_state(api_key, did, network, chain)
            current_height = node_state["height"]
        return current_height

    # Common method for mainchain, did sidechain and token sidechain
    def get_current_node_state(self, api_key, did, network, chain):
        return self.rpc_method(api_key, did, network, chain, "getnodestate", {})

    def rpc_method(self, api_key, did, network, chain, method, params):
        jwt_info = {
            'network': network,
            'chain': chain,
            'method': method,
            'params': params
        }

        jwt_token = jwt.encode({
            'jwt_info': jwt_info,
            'exp': datetime.datetime.utcnow() + datetime.timedelta(hours=TOKEN_EXPIRATION)
        }, api_key, algorithm='HS256')

        response = self.stub.RpcMethod(node_rpc_pb2.Request(input=jwt_token), timeout=REQUEST_TIMEOUT, metadata=[('did', did)])
        data = ''

        if response.status:
            output = jwt.decode(response.output, key=api_key, algorithms=['HS256']).get('jwt_info')
            data = output['result']

        return data




# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: node_rpc.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='node_rpc.proto',
  package='node_rpc',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x0enode_rpc.proto\x12\x08node_rpc\"\x18\n\x07Request\x12\r\n\x05input\x18\x01 \x01(\t\"B\n\x08Response\x12\x0e\n\x06output\x18\x01 \x01(\t\x12\x16\n\x0estatus_message\x18\x02 \x01(\t\x12\x0e\n\x06status\x18\x03 \x01(\x08\x32?\n\x07NodeRpc\x12\x34\n\tRpcMethod\x12\x11.node_rpc.Request\x1a\x12.node_rpc.Response\"\x00\x62\x06proto3')
)




_REQUEST = _descriptor.Descriptor(
  name='Request',
  full_name='node_rpc.Request',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='input', full_name='node_rpc.Request.input', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=28,
  serialized_end=52,
)


_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='node_rpc.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='output', full_name='node_rpc.Response.output', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status_message', full_name='node_rpc.Response.status_message', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='node_rpc.Response.status', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=54,
  serialized_end=120,
)

DESCRIPTOR.message_types_by_name['Request'] = _REQUEST
DESCRIPTOR.message_types_by_name['Response'] = _RESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Request = _reflection.GeneratedProtocolMessageType('Request', (_message.Message,), {
  'DESCRIPTOR' : _REQUEST,
  '__module__' : 'node_rpc_pb2'
  # @@protoc_insertion_point(class_scope:node_rpc.Request)
  })
_sym_db.RegisterMessage(Request)

Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
  'DESCRIPTOR' : _RESPONSE,
  '__module__' : 'node_rpc_pb2'
  # @@protoc_insertion_point(class_scope:node_rpc.Response)
  })
_sym_db.RegisterMessage(Response)



_NODERPC = _descriptor.ServiceDescriptor(
  name='NodeRpc',
  full_name='node_rpc.NodeRpc',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=122,
  serialized_end=185,
  methods=[
  _descriptor.MethodDescriptor(
    name='RpcMethod',
    full_name='node_rpc.NodeRpc.RpcMethod',
    index=0,
    containing_service=None,
    input_type=_REQUEST,
    output_type=_RESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_NODERPC)

DESCRIPTOR.services_by_name['NodeRpc'] = _NODERPC

# @@protoc_insertion_point(module_scope)



import pytest
import json
from decouple import Config, RepositoryEnv, config

from elastos_adenine.stubs import health_check_pb2
from elastos_adenine.health_check import HealthCheck
from elastos_adenine.sidechain_eth import SidechainEth
from elastos_adenine.common import Common

test_input = {}
env_config = None


@pytest.fixture(scope="module")
def set_test_input(request):
    global env_config

    DOTENV_FILE = '.env.test'
    env_config = Config(RepositoryEnv(DOTENV_FILE))

    test_input['host'] = config('GRPC_SERVER_HOST')
    test_input['port'] = config('GRPC_SERVER_PORT')
    test_input['production'] = config('PRODUCTION', default=False, cast=bool)
    test_input['network'] = env_config('NETWORK')
    test_input['ela_to_use'] = env_config('ELA_TO_USE')
    test_input['ela_eth_to_use'] = env_config('ELA_ETH_TO_USE')
    test_input['did_to_use'] = env_config('DID_TO_USE')
    test_input['private_key_to_use'] = env_config('PRIVATE_KEY_TO_USE')
    return test_input


@pytest.mark.dependency()
def test_health_check(set_test_input):
    # Health Check
    try:
        health_check = HealthCheck(test_input['host'], test_input['port'], test_input['production'])
        response = health_check.check()
        assert response.status == health_check_pb2.HealthCheckResponse.SERVING, "grpc server is not running properly"
    except Exception as e:
        health_check_status = False
        assert health_check_status == True, f"grpc server is not running properly: {e}"


@pytest.mark.dependency(depends=["test_health_check"])
def test_get_api_key():
    # Get API Key
    common = Common(test_input['host'], test_input['port'], test_input['production'])
    response = common.get_api_key_request(config('SHARED_SECRET_ADENINE'), test_input['did_to_use'])
    if response['status']:
        json_output = json.loads(response['output'])
        for i in json_output['result']:
            if i == 'api_key':
                test_input['api_key_to_use'] = json_output['result'][i]
    assert response['status'] == True, "In Get Api Key-> " + response['status_message']
    assert len(test_input['api_key_to_use']) == 64, "Testing API Key length Failed"


@pytest.mark.dependency(depends=["test_health_check"])
def test_deploy_eth_contract():
    sidechain_eth = SidechainEth(test_input['host'], test_input['port'], test_input['production'])
    response = sidechain_eth.deploy_eth_contract(test_input['api_key_to_use'], test_input['did_to_use'],
                                                 test_input['network'],
                                                 test_input['ela_eth_to_use'],
                                                 '0x35a12175385b24b2f906d6027d440aac7bd31e1097311fa8e3cf21ceac7c4809',
                                                 2000000, 'test/HelloWorld.sol')
    if response['status']:
        json_output = json.loads(response['output'])
        for i in json_output['result']:
            assert json_output['result'][i] is not None, "In Deploy Eth Contract-> " + response['status_message']
    assert response['status'] == True, "In Deploy Eth Contract-> " + response['status_message']


@pytest.mark.dependency(depends=["test_health_check"])
def test_watch_eth_contract():
    sidechain_eth = SidechainEth(test_input['host'], test_input['port'], test_input['production'])
    response = sidechain_eth.watch_eth_contract(test_input['api_key_to_use'], test_input['did_to_use'],
                                                test_input['network'],
                                                '0xc0ba7D9CF73c0410FfC9FB5b768F5257906B13c1', 'HelloWorld',
                                                'QmXYqHg8gRnDkDreZtXJgqkzmjujvrAr5n6KXexmfTGqHd')
    if response['status']:
        json_output = json.loads(response['output'])
        for i in json_output['result']:
            assert json_output['result'][i] is not None, "In Watch Eth Contract-> " + response['status_message']
    assert response['status'] == True, "In Watch Eth Contract-> " + response['status_message']



from elastos_adenine.health_check import HealthCheck
from elastos_adenine.node_rpc import NodeRpc
from elastos_adenine.common import Common
from elastos_adenine.hive import Hive
from elastos_adenine.sidechain_eth import SidechainEth
from elastos_adenine.wallet import Wallet



import grpc
import jwt
import datetime
import json

from .stubs import common_pb2, common_pb2_grpc
from elastos_adenine.settings import REQUEST_TIMEOUT, TOKEN_EXPIRATION


class Common:

    def __init__(self, host, port, production):
        if not production:
            self._channel = grpc.insecure_channel('{}:{}'.format(host, port))
        else:
            credentials = grpc.ssl_channel_credentials()
            self._channel = grpc.secure_channel('{}:{}'.format(host, port), credentials)
        
        self.stub = common_pb2_grpc.CommonStub(self._channel)

    def close(self):
        self._channel.close()

    def generate_api_request(self, secret_key, did):
        jwt_info = {}

        jwt_token = jwt.encode({
            'jwt_info': jwt_info,
            'exp': datetime.datetime.utcnow() + datetime.timedelta(hours=TOKEN_EXPIRATION)
        }, secret_key, algorithm='HS256')

        response = self.stub.GenerateAPIRequest(common_pb2.Request(input=jwt_token), timeout=REQUEST_TIMEOUT, metadata=[('did', did)])
        
        if response.status:
            output = jwt.decode(response.output, key=secret_key, algorithms=['HS256']).get('jwt_info')
            result = {
                'output': json.dumps(output),
                'status_message': response.status_message,
                'status': response.status
            }
        else:
            result = {
                'output': '',
                'status_message': response.status_message,
                'status': response.status
            }
        return result

    def get_api_key_request(self, secret_key, did):
        jwt_info = {}

        jwt_token = jwt.encode({
            'jwt_info': jwt_info,
            'exp': datetime.datetime.utcnow() + datetime.timedelta(hours=TOKEN_EXPIRATION)
        }, secret_key, algorithm='HS256')

        response = self.stub.GetAPIKey(common_pb2.Request(input=jwt_token), timeout=REQUEST_TIMEOUT, metadata=[('did', did)])
        
        if response.status:
            output = jwt.decode(response.output, key=secret_key, algorithms=['HS256']).get('jwt_info')
            result = {
                'output': json.dumps(output),
                'status_message': response.status_message,
                'status': response.status
            }
        else:
            result = {
                'output': '',
                'status_message': response.status_message,
                'status': response.status
            }
        return result



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from . import wallet_pb2 as wallet__pb2


class WalletStub(object):
  """The service definition.
  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.CreateWallet = channel.unary_unary(
        '/wallet.Wallet/CreateWallet',
        request_serializer=wallet__pb2.Request.SerializeToString,
        response_deserializer=wallet__pb2.Response.FromString,
        )
    self.RequestELA = channel.unary_unary(
        '/wallet.Wallet/RequestELA',
        request_serializer=wallet__pb2.Request.SerializeToString,
        response_deserializer=wallet__pb2.Response.FromString,
        )


class WalletServicer(object):
  """The service definition.
  """

  def CreateWallet(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def RequestELA(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_WalletServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'CreateWallet': grpc.unary_unary_rpc_method_handler(
          servicer.CreateWallet,
          request_deserializer=wallet__pb2.Request.FromString,
          response_serializer=wallet__pb2.Response.SerializeToString,
      ),
      'RequestELA': grpc.unary_unary_rpc_method_handler(
          servicer.RequestELA,
          request_deserializer=wallet__pb2.Request.FromString,
          response_serializer=wallet__pb2.Response.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'wallet.Wallet', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



import grpc
from decouple import config

from .stubs import health_check_pb2, health_check_pb2_grpc


class HealthCheck:

    def __init__(self, host, port, production):
        if not production:
            self._channel = grpc.insecure_channel('{}:{}'.format(host, port))
        else:
            credentials = grpc.ssl_channel_credentials()
            self._channel = grpc.secure_channel('{}:{}'.format(host, port), credentials)

        self.stub = health_check_pb2_grpc.HealthStub(self._channel)

    def close(self):
        self._channel.close()

    def check(self):
        request = health_check_pb2.HealthCheckRequest()
        response = self.stub.Check(request)
        return response
